      program advection
      implicit none
!
!     program to compute the analytic solution of the
!     linear 1d advection equation: 
!
!     dT/dt=-u dT/dx
!
      integer,parameter :: nx = 100 ! x-dimension
      integer,parameter :: nt = 200 ! t-dimension
      real :: t(1:nx)               ! T
!
      integer :: nk = 1             ! wave number
      real ::  u = 10.              ! velocity
      real :: dt = 1.0              ! time step
      real :: dx = 1.0              ! grid point distance in space
      real :: t0 = 10.              ! initial T amplitude
!
      integer :: jx,jt              ! loop indizes
      real :: time                  ! time
      real :: rx                    ! length of the area
      real :: rk                    ! real wave number
      real :: pi                    ! Pi
!
!     open output file (unformatted) 
!
      open(10,file='T.dat',form='unformatted')
!
!     set constants
! 
      pi=2.*asin(1.)
      rx=nx*dx                        
      rk=2.*pi*real(nk)/rx        
!
!     initialize 
!
      time=0
      do jx=1,nx
       t(jx)=t0*sin(rk*jx*dx)
      enddo
!
!     write initial T
!
      write(10) t(1:nx)
!
!     time loop
!
      do jt=1,nt
       time=time+dt
       do jx=1,nx
        t(jx)=t0*sin(rk*(jx*dx-u*time))
       enddo
       write(10) t(1:nx)
      enddo
!
!     close file
!
      close(10)
!
      stop
      end
