      program advection
      implicit none
!
!     program to compute some numeric solutions of the
!     linear 1d advection equation:
!
!     dT/dt=-u dT/dx
!
!     for advection of a 'box'
!
      integer,parameter :: ns = 3   ! no solutions
!
!     1: upstream
!     2: leap frog
!     3: leap frog w. Robert-Asselin filter
!
      integer,parameter :: nx = 100 ! x-dimension
      integer,parameter :: nt = 200 ! t-dimension
!
      real :: to(0:nx+1,ns)        ! T(t-1)  
      real :: t(0:nx+1,ns)         ! T(t)
      real :: tn(0:nx+1,ns)        ! T(t+1)
!
      real ::  u = 10.              ! velocity
      real :: dt = 0.05             ! time step
      real :: dx = 1.               ! grid point distance in space
      real :: t0 = 10.              ! initial u amplitude
!
      integer :: jx,jt,js           ! loop indizes
      real :: time                  ! time
      real :: rx                    ! length of the area
      real :: x                     ! actual x
      real :: cr                    ! courant number
      real :: gamma = 0.1           ! R-A filter const.
      real :: pi                    ! Pi
   
!
!     open ouput files
!
!     a) grads input
!
      open(10,file='T_adv_box.dat',form='unformatted')   
!
!     a) gnuplot input
!
      open(11,file='T_adv_box_t.txt',form='formatted')  ! time series
      open(12,file='T_adv_box_xa.txt',form='formatted') ! x-series t=0
      open(13,file='T_adv_box_xb.txt',form='formatted') ! x-series t=199
!
!     set constants
! 
      pi=2.*asin(1.)                
      rx=nx*dx                       
      cr=u*dt/dx
!
!     initialize 
!
      time=0
      do jx=1,nx
       x=(jx-1)*dx
       if(x < 50.) then
        t(jx,:)=10.
       else
        t(jx,:)=0.
       endif
      enddo
!
!     cyclic boundary conditions
!
      t(0,:)=t(nx,:)
      t(nx+1,:)=t(1,:)
      to(:,:)=t(:,:)
!
!     write output
!
      do js=1,ns
       write(10) t(1:nx,js)
      enddo
      write(11,*) 0,t(1,:)
      do jx=1,nx
       write(12,*) (jx-1)*dx,t(jx,:)
      enddo 
!
!     do the time integration
!
      do jt=1,nt
       time=time+dt
       do jx=1,nx
        x=(jx-1)*dx
        tn(jx,1)=t(jx,1)-cr*(t(jx,1)-t(jx-1,1))       ! upstream
        if(jt==1) then
         tn(jx,2:3)=tn(jx,1)                           ! lf initialization t=1
        else
         tn(jx,2)=to(jx,2)-cr*(t(jx+1,2)-t(jx-1,2))    ! leap frog
         tn(jx,3)=to(jx,3)-cr*(t(jx+1,3)-t(jx-1,3))    ! leap frog w filter
        endif
       enddo
       tn(0,:)=tn(nx,:)
       tn(nx+1,:)=tn(1,:)
       to(:,1:2)=t(:,1:2)
       t(:,1:2)=tn(:,1:2)
!
!      Robert-Asselin filtering
!
       to(:,3)=t(:,3)+gamma*(tn(:,3)+to(:,3)-2.*t(:,3))
       t(:,3)=tn(:,3)
!
!      write output
!
       do js=1,ns
        write(10) t(1:nx,js)
       enddo
       write(11,*) time,t(1,:)
       if(jt==199) then
        do jx=1,nx
         write(13,*) (jx-1)*dx,t(jx,:)
        enddo
       endif
      enddo
!
!     close files
!
      close(10)
      close(11)
      close(12)
      close(13)
!
      stop
      end
