      program advection
      implicit none
!
!     program to compute the numeric solutions of the
!     linear 1d advection equation:
!
!     dT/dt=-u dT/dx
!
!     using a semi-Lagrangian scheme
!
      integer,parameter :: nx = 100 ! x-dimension
      integer,parameter :: nt = 200 ! t-dimension
!
      real :: t(0:nx+1)             ! T(t)
      real :: tn(0:nx+1)            ! T(t+1)
      real :: x(0:nx+1)             ! x
!
      real ::  u = 10.              ! velocity
      real :: dt = 0.05             ! time step
      real :: dx = 1.               ! grid point distance in space
      real :: t0 = 10.              ! initial T amplitude
!
      integer :: jx,jt,js       ! loop indizes
      real :: time                  ! time
      real :: rx                    ! length of the area
      real :: xo                    ! old x of parcel      
      real :: gw1,gw2               ! weights for interpolation
      real :: pi                    ! Pi
   
!
!     open ouput files
!
!     a) grads input
!
      open(10,file='T_adv_sl.dat',form='unformatted')   
!
!     a) gnuplot input
!
      open(11,file='T_adv_sl_t.txt',form='formatted')  ! time series
      open(12,file='T_adv_sl_xa.txt',form='formatted') ! x-series t=0
      open(13,file='T_adv_sl_xb.txt',form='formatted') ! x-series t=199
!
!     set constants
! 
      pi=2.*asin(1.)                
      rx=nx*dx                       
!
!     initialize 
!
      time=0
      do jx=1,nx
       x(jx)=(jx-1)*dx
       if(x(jx) < 50.) then
        t(jx)=10.
       else
        t(jx)=0.
       endif
      enddo
!
!     cyclic boundary conditions
!
      t(0)=t(nx)
      t(nx+1)=t(1)
      x(0)=2.*x(1)-x(2)
      x(nx+1)=2.*x(nx)-x(nx-1)
!
!     write output
!
      write(10) t(1:nx)
      write(11,*) 0,t(1)
      do jx=1,nx
       write(12,*) (jx-1)*dx,t(jx)
      enddo 
!
!     do the time integration
!
      do jt=1,nt
       time=time+dt
       do jx=1,nx
!
!      compute old x
!
        xo=x(jx)-u*dt
!
!       check if x(jx-1) <= xo <= x(jx+1) (otherweise cfl error)
!
        if(xo < x(jx-1) .or. xo > x(jx+1)) then
         print*,'CFL ERROR at jx= ',jx
         stop 'ERROR'
        endif
!
!       do the interpolation
!  
        if(xo >= x(jx)) then
         gw1=x(jx+1)-xo
         gw2=xo-x(jx)
         tn(jx)=(gw1*t(jx)+gw2*t(jx+1))/(gw1+gw2)
        else
         gw1=x(jx)-xo
         gw2=xo-x(jx-1)
         tn(jx)=(gw1*t(jx-1)+gw2*t(jx))/(gw1+gw2)
        endif
       enddo
       tn(0)=tn(nx)
       tn(nx+1)=tn(1)
       t(:)=tn(:)
!
!      write output
!
       write(10) t(1:nx)
       write(11,*) time,t(1)
       if(jt==199) then
        do jx=1,nx
         write(13,*) (jx-1)*dx,t(jx)
        enddo
       endif
      enddo
!
!     close files
!
      close(10)
      close(11)
      close(12)
      close(13)
!
      stop
      end
