      program advection
      implicit none
!
!     program to compute a numeric solutions of the
!     linear 1d Burgers equation:
!
!     du/dt=-u du/dx + K d2(u)/dx2
!
!     for different K
!
      integer,parameter :: ns = 3   ! no solutions
!
!     1: K=0
!     2: K=1
!     3: K=5
!
      integer,parameter :: nx = 100 ! x-dimension
      integer,parameter :: nt = 500 ! t-dimension
!
      real :: uo(-1:nx+2,ns)        ! u(t-1)  
      real :: u(-1:nx+2,ns)         ! u(t)
      real :: un(-1:nx+2,ns)        ! u(t+1)
!
      integer :: nk = 1             ! wave number (integer, see below)
      real :: dt = 0.05             ! time step
      real :: dx = 1.               ! grid point distance in space
      real :: u0 = 1.               ! initial u amplitude
      real :: diff1 = 0.            ! diffusion coef K sol. 1
      real :: diff2 = 1.            ! diffusion coef K sol. 2
      real :: diff3 = 5.            ! diffusion coef K sol. 3
!
      integer :: jx,jt,js           ! loop indizes
      real :: time                  ! time
      real :: dt2                   ! 2*dt
      real :: rx                    ! length of the area
      real :: rk                    ! real wave number (incl 2*pi)
      real :: x                     ! actual x
      real :: cr                    ! courant number
      real :: pi                    ! Pi
      real :: zd                    ! u-tendency due to diffusion 
!
!     open ouput files
!
!     a) grads input
!
      open(10,file='u_adv.dat',form='unformatted')   
!
!     a) gnuplot input
!
      open(11,file='u_adv_t.txt',form='formatted')  ! time series
      open(12,file='u_adv_xa.txt',form='formatted') ! x-series t=0
      open(13,file='u_adv_xb.txt',form='formatted') ! x-series t=199
!
!     set constants
! 
      dt2=2.*dt
      pi=2.*asin(1.)                
      rx=nx*dx
      rk=2.*pi*real(nk)/rx
!
!     initialize 
!
      time=0
      do jx=1,nx
       x=(jx-1)*dx
       u(jx,:)=u0*sin(rk*x)
      enddo
!
!     cyclic boundary conditions
!
      u(0,:)=u(nx,:)
      u(-1,:)=u(nx-1,:)
      u(nx+1,:)=u(1,:)
      u(nx+2,:)=u(2,:)
      uo(:,:)=u(:,:)
!
!     write output
!
      do js=1,ns
       write(10) u(1:nx,js)
      enddo
      write(11,*) 0,u(1,:)
      do jx=1,nx
       write(12,*) (jx-1)*dx,u(jx,:)
      enddo 
!
!     do the time integration
!
!     one foreward to get u(t)
!
      do jx=1,nx
       zd=dt*diff1*(uo(jx+1,1)+uo(jx-1,1)-2.*uo(jx,1))/dx/dx
       u(jx,1)=uo(jx,1)                                                 &
     &        -dt*(u(jx+2,1)*u(jx+1,1)-u(jx-1,1)*u(jx-2,1))/(6.*dx)+zd
       zd=dt*diff2*(uo(jx+1,2)+uo(jx-1,2)-2.*uo(jx,2))/dx/dx
       u(jx,2)=uo(jx,2)                                                 &
     &        -dt*(u(jx+2,2)*u(jx+1,2)-u(jx-1,2)*u(jx-2,2))/(6.*dx)+zd
       zd=dt*diff3*(uo(jx+1,3)+uo(jx-1,3)-2.*uo(jx,3))/dx/dx
       u(jx,3)=uo(jx,3)                                                 &
     &        -dt*(u(jx+2,3)*u(jx+1,3)-u(jx-1,3)*u(jx-2,3))/(6.*dx)+zd

      enddo
      u(0,:)=u(nx,:)
      u(-1,:)=u(nx-1,:)
      u(nx+1,:)=u(1,:)
      u(nx+2,:)=u(2,:)
!
!     normal Leapfrog/FTCS
!
      do jt=1,nt
       time=time+dt
       do jx=1,nx
        x=(jx-1)*dx
        zd=dt2*diff1*(uo(jx+1,1)+uo(jx-1,1)-2.*uo(jx,1))/dx/dx
        un(jx,1)=uo(jx,1)                                               &
     &          -dt2*(u(jx+2,1)*u(jx+1,1)-u(jx-1,1)*u(jx-2,1))/(6.*dx)+zd
        zd=dt2*diff2*(uo(jx+1,2)+uo(jx-1,2)-2.*uo(jx,2))/dx/dx
        un(jx,2)=uo(jx,2)                                               &
     &          -dt2*(u(jx+2,2)*u(jx+1,2)-u(jx-1,2)*u(jx-2,2))/(6.*dx)+zd
        zd=dt2*diff3*(uo(jx+1,3)+uo(jx-1,3)-2.*uo(jx,3))/dx/dx
        un(jx,3)=uo(jx,3)                                               &
     &          -dt2*(u(jx+2,3)*u(jx+1,3)-u(jx-1,3)*u(jx-2,3))/(6.*dx)+zd
       enddo
       un(0,:)=u(nx,:)
       un(-1,:)=u(nx-1,:)
       un(nx+1,:)=un(1,:)
       un(nx+2,:)=un(2,:)
       uo(:,:)=u(:,:)
       u(:,:)=un(:,:)
!
!      write output
!
       do js=1,ns
        write(10) u(1:nx,js)
       enddo
       write(11,*) time,u(1,:)
       if(jt==500) then
        do jx=1,nx
         write(13,*) (jx-1)*dx,u(jx,:)
        enddo
       endif
      enddo
!
!     close files
!
      close(10)
      close(11)
      close(12)
      close(13)
!
      stop
      end
