      program advection
      implicit none
!
!     program to compute the analytic and some numeric solutions of the
!     linear 1d transport equation:
!
!     dT/dt=-u dT/dx+ K d2(T)/dx2
!
      integer,parameter :: ns = 3   ! no solutions
!
!     1: analytic
!     2: leap frog/FTCS
!     3: leap frog/FTCS w. Robert-Asselin filter
!
      integer,parameter :: nx = 100 ! x-dimension
      integer,parameter :: nt = 200 ! t-dimension
!
      real :: to(0:nx+1,ns)         ! T(t-1)  (needed for leapfrog)
      real :: t(0:nx+1,ns)          ! T(t)
      real :: tn(0:nx+1,ns)         ! T(t+1)
!
      integer :: nk = 1             ! wave number (integer, see below)
      real ::  u = 10.              ! velocity
      real :: dt = 0.05             ! time step
      real :: dx = 1.               ! grid point distance in space
      real :: t0 = 10.              ! initial T amplitude
      real :: diff = 2.             ! diffusion coef.
      real :: gamma = 0.1           ! R-A filter const.
!
      integer :: jx,jt,js           ! loop indizes
      real :: time                  ! time
      real :: rx                    ! length of the area
      real :: x                     ! actual x
      real :: rk                    ! real wave number (incl 2*pi)
      real :: cr                    ! courant number
      real :: pi                    ! Pi
   
!
!     open ouput files
!
!     a) grads input
!
      open(10,file='T_trans.dat',form='unformatted')   
!
!     a) gnuplot input
!
      open(11,file='T_trans_t.txt',form='formatted')  ! time series
      open(12,file='T_trans_xa.txt',form='formatted') ! x-series t=0
      open(13,file='T_trans_xb.txt',form='formatted') ! x-series t=199
!
!     set constants
! 
      pi=2.*asin(1.)                
      rx=nx*dx                       
      rk=2.*pi*real(nk)/rx 
      cr=u*dt/dx
!
!     initialize 
!
      time=0
      do jx=1,nx
       x=(jx-1)*dx
       t(jx,:)=t0*sin(rk*x)
      enddo
!
!     cyclic boundary conditions
!
      t(0,:)=t(nx,:)
      t(nx+1,:)=t(1,:)
      to(:,:)=t(:,:)
!
!     write output
!
      do js=1,ns
       write(10) t(1:nx,js)
      enddo
      write(11,*) 0,t(1,:)
      do jx=1,nx
       write(12,*) (jx-1)*dx,t(jx,:)
      enddo 
!
!     do the time integration
!
      do jt=1,nt
       time=time+dt
       do jx=1,nx
        x=(jx-1)*dx
        tn(jx,1)=t0*sin(rk*(x-u*time))*exp(-time*diff*rk*rk) ! analytical
        if(jt > 1) then
!
!       leap frog
!
         tn(jx,2)=to(jx,2)-cr*(t(jx+1,2)-t(jx-1,2))                     &
     &           +2.*diff*(to(jx-1,2)+to(jx+1,2)-2.*to(jx,2))*dt/dx/dx 
!
!       lf with filter

         tn(jx,3)=to(jx,3)-cr*(t(jx+1,3)-t(jx-1,3))                     &
     &           +2.*diff*(to(jx-1,3)+to(jx+1,3)-2.*to(jx,3))*dt/dx/dx
        else
         tn(jx,2)=tn(jx,1)
         tn(jx,3)=tn(jx,1)
        endif 
       enddo
       tn(0,:)=tn(nx,:)
       tn(nx+1,:)=tn(1,:)
       to(:,1:2)=t(:,1:2)
       t(:,1:2)=tn(:,1:2)
!
!      Robert-Asselin filtering
!
       to(:,3)=t(:,3)+gamma*(tn(:,3)+to(:,3)-2.*t(:,3))
       t(:,3)=tn(:,3)
!
!      write output
!
       do js=1,ns
        write(10) t(1:nx,js)
       enddo
       write(11,*) time,t(1,:)
       if(jt==199) then
        do jx=1,nx
         write(13,*) (jx-1)*dx,t(jx,:)
        enddo
       endif
      enddo
!
!     close files
!
      close(10)
      close(11)
      close(12)
      close(13)
!
      stop
      end
